package provide form 0.99.5

##############################################################################
# form.tcl - simple package for editing records stored in arrays.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################

# Forms are simple windows which display a single column of
# labels and entries. The information put in a form is meant to
# be stored as a list in an array. The first entry of the form
# is the index into the array.

# save_command and delete_command must be defined as
# appropriate to what is to be done with the form info
# An application using the form package should just call
# form_menu


# labels is a list of the labels to the entry widgets
# save_command is a call_back that's given a list of each 
# element of labels as an argument
proc form {name labels save_command} {

    if {[winfo exists .$name]} {
	raise .$name
	return
    }

    toplevel .$name
    wm title .$name $name

    button .$name.save -text "Save" -command "\
	    form_save $save_command $name;destroy .$name"
    button .$name.exit -text "Exit" -command "destroy .$name"
    
    set count 0
    foreach label $labels {
	label .$name.l$count -text $label
	entry .$name.e$count  
	grid  .$name.l$count -row $count -column 0 -sticky w
	grid  .$name.e$count -row $count -column 1 -sticky ew
	incr count
    }
    
    grid  .$name.save -row $count -column 0 -pady 5
    grid  .$name.exit -row $count -column 1 -pady 5
    grid columnconfigure .$name 1 -weight 1
    wm geometry .$name +[winfo pointerx .]+[winfo pointery .]
    wm resizable .$name 1 0
}

# puts the elements of list in form name
proc form_put {name list} {
    set count 0

    foreach element $list {
	.$name.e$count delete 0 end  
	.$name.e$count insert 0 $element
	incr count
    }
}

# returns a list of the current values in name's entry widgets
proc form_get {name} {
    set number [expr {([llength [winfo children .$name]]-2)/2}]

    for {set count 0} {$count < $number} {incr count} {
	lappend results [.$name.e$count get]
    }
    return $results
}

# save the record of form name  with a callback command.
proc form_save {command name} {   
    eval $command [form_get $name]
    fmenu_update $name
}

# delete a record of form name with the callback command.
# the delete command is given a index in the record's array
# to delete.
proc form_delete {command name} {
    set i [.fm$name.list curselection]

    if {$i!=""} {
	$command [.fm$name.list get $i]
	fmenu_update $name
    }
}

# fills the form with the active record
proc form_update {name} {
    global [set name]_array

    set i [.fm$name.list curselection]
    
    if {$i!=""} {
	set list "[list [.fm$name.list get $i]] \
                  [set [set name]_array([.fm$name.list get $i])]"
	form_put $name $list
    }
}

# updates the form menu to reflect the record array of name
proc fmenu_update {name} {
    global [set name]_array

    .fm$name.list delete 0 end
    foreach element [lsort [array names [set name]_array]] {
	.fm$name.list insert end $element
    }
}

# a menu of records of type name
proc form_menu {name labels save_command delete_command} {

    if {[winfo exists .fm$name]} {
	raise .fm$name
	return
    }
    
    toplevel .fm$name
    wm title .fm$name "Edit $name"

    button .fm$name.delete -text "Delete" -command \
	[list form_delete $delete_command $name]
    
    button .fm$name.edit -text "Edit" -command "\
	    form $name $labels $save_command;\
	    form_update $name"    
    button .fm$name.exit -text "Exit" -command "\
	    catch {destroy .fm$name .$name}"
    button .fm$name.new -text "New" -command \
	"form $name $labels $save_command"

    scrollbar .fm$name.yscroll -command ".fm$name.list yview"
    listbox .fm$name.list -selectmode single \
	-yscroll ".fm$name.yscroll set"

    grid .fm$name.edit -row 0 -column 0 
    grid .fm$name.delete -row 0 -column 1
    grid .fm$name.list -row 1 -column 0 -columnspan 2 -sticky "news"
    grid .fm$name.yscroll -row 1 -column 2 -sticky "ns"
    grid .fm$name.new -row 2 -column 0
    grid .fm$name.exit -row 2 -column 1
    grid rowconfigure .fm$name 1 -weight 1
    grid columnconfigure .fm$name 1 -weight 1
    
    fmenu_update $name
    bind .fm$name.list <Double-Button-1> "\
	     form $name $labels $save_command;\
	     form_update $name"

    wm geometry .fm$name +[winfo pointerx .]+[winfo pointery .]
}
